// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESWIN_ACCELERATOR_KIT_API_H__
#define __ESWIN_ACCELERATOR_KIT_API_H__

#include "es_type.h"
#include "es_ak_types.h"
#include "es_ak_error.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

/**
 * @brief Get Accelerator version information.
 *
 * @param[out] version Version number of the Accelerator.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_GetVersion(ES_U64 *version);

/**
 * @brief Get Accelerator version string.
 *
 * @param[out] version Buffer to store the version information
 * @param[in] maxSize Maximum size of the buffer
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_AK_GetVersionString(ES_CHAR *version, ES_U32 maxSize);

/**
 * @brief Get Accelerator capability
 *
 * @param[out] capability Structure to store Accelerator capability information
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_AK_GetCapability(ES_AK_Capability_S *capability);

/**
 * @brief Set the log level for Accelerator logging.
 *
 * @param[in] level Log level for Accelerator logging.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_SetLogLevel(ES_U32 level);

/**
 * @brief Initializes device resources, including opening hardware devices like DSP.
 *
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_Init();

/**
 * @brief Releases device resources and closes hardware devices.
 *
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_Deinit();

/**
 * @brief Configures the device for computation, such as specifying a DSP for computation.
 *
 * @param[in] devices An array of ES_AK_DEVICE_E specifying the device(s) for computation.
 * @param[in] devNum The number of devices specified in the devices array.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_SetDevice(const ES_AK_DEVICE_E *devices, ES_U32 devNum);

/**
 * @brief Retrieves the current computation device.
 *
 * @param[out] devices An array of ES_AK_DEVICE_E to store the current computation device(s).
 * @param[in] devNum The number of devices specified in the devices array.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_GetDevice(ES_AK_DEVICE_E *devices, ES_U32 devNum);

/**
 * @brief Computes the cosine distance between two sets of data.
 *
 * @param[in] inputBase  Information of input data for computing cosine distance.
 * @param[in] inputQuery Information of input data for computing cosine distance.
 * @param[in] output     Cosine distance information between inputBase and inputQuery.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_DSP_CosDistance(ES_TENSOR_S inputBase, ES_TENSOR_S inputQuery, ES_TENSOR_S output);

/**
 * @brief Applies a perspective affine transformation to the input tensor.
 *
 * @param[in] input The input tensor to be transformed.
 * @param[in] output The output tensor containing the transformed data.
 * @param[in] M The perspective transformation matrix, size should be AFFINE_MATRIX_SIZE.
 * @param[in] flag The interpolation method flag.
 * @param[in] borderMode The border mode for handling out of range values.
 * @param[in] borderValue The value used for out of range pixels.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_DSP_PerspectiveAffine(ES_TENSOR_S input, ES_TENSOR_S output, ES_FLOAT M[AFFINE_MATRIX_SIZE],
                                   ES_INTER_FLAG_E flag, ES_BORDER_TYPES_E borderMode, ES_U8 borderValue);

/**
 * @brief Applies a warp affine transformation to the input tensor.
 *
 * @param[in] input The input tensor to be transformed.
 * @param[in] output The output tensor containing the transformed data.
 * @param[in] M The warp transformation matrix, size should be AFFINE_MATRIX_SIZE.
 * @param[in] flag The interpolation method flag.
 * @param[in] borderMode The border mode for handling out of range values.
 * @param[in] borderValue The value used for out of range pixels.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_DSP_WarpAffine(ES_TENSOR_S input, ES_TENSOR_S output, ES_FLOAT M[AFFINE_MATRIX_SIZE],
                            ES_INTER_FLAG_E flag, ES_BORDER_TYPES_E borderMode, ES_U8 borderValue);

/**
 * @brief Performs object detection and outputs the detection results.
 *
 * @param[in] inFeaturesPtr Pointer to an array of input feature tensors.
 * @param[in] inFeaturesNum Number of input feature tensors.
 * @param[in] outBoxInfo The output tensor containing the bounding box information.
 * @param[in] outBoxNum The output tensor containing the number of detected boxes.
 * @param[in] detectNet The detection network type.
 * @param[in] anchorsNum Number of anchor boxes.
 * @param[in] anchorScale Array of anchor scales.
 * @param[in] imgH Height of the input image.
 * @param[in] imgW Width of the input image.
 * @param[in] clsNum Number of object classes.
 * @param[in] inputScale Array of input tensor scales.
 * @param[in] nmsMethod The non-maximum suppression (NMS) method.
 * @param[in] iouMethod The intersection over union (IOU) method.
 * @param[in] outBoxType The output box type.
 * @param[in] coordNorm Whether the box coordinates are normalized.
 * @param[in] maxBoxesPerClass Maximum number of boxes per class.
 * @param[in] maxBoxesPerBatch Maximum number of boxes per batch.
 * @param[in] scoreThreshold Score threshold for filtering detections.
 * @param[in] iouThreshold IOU threshold for NMS.
 * @param[in] softNmsSigma Sigma value for soft NMS.
 * @param[in] effecImgOffsetX Effective offset in X direction for box coordinates.
 * @param[in] effecImgOffsetY Effective offset in Y direction for box coordinates.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_DSP_DetectionOut(ES_TENSOR_S *inFeaturesPtr, ES_S32 inFeaturesNum, ES_TENSOR_S outBoxInfo,
                              ES_TENSOR_S outBoxNum, ES_DET_NETWORK_E detectNet, ES_S32 anchorsNum,
                              ES_FLOAT anchorScale[MAX_TOTAL_ANCHORS_NUM * 2], ES_S32 imgH, ES_S32 imgW, ES_S32 clsNum,
                              ES_FLOAT inputScale[MAX_IN_TENSOR_NUM], ES_NMS_METHOD_E nmsMethod,
                              ES_IOU_METHOD_E iouMethod, ES_BOX_TYPE_E outBoxType, ES_BOOL coordNorm,
                              ES_S32 maxBoxesPerClass, ES_S32 maxBoxesPerBatch, ES_FLOAT scoreThreshold,
                              ES_FLOAT iouThreshold, ES_FLOAT softNmsSigma, ES_FLOAT effecImgOffsetX,
                              ES_FLOAT effecImgOffsetY);

/**
 * @brief Computes the indices of the k largest elements along a specified axis of the input tensor.
 *
 * @param[in] input The input tensor.
 * @param[in] output The output tensor containing the k largest elements.
 * @param[in] outputIdx The output tensor containing the indices of the k largest elements.
 * @param[in] k The number of top elements to retain.
 * @param[in] axis The axis along which to compute the indices.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_DSP_Argmax(ES_TENSOR_S input, ES_TENSOR_S output, ES_TENSOR_S outputIdx, ES_S32 k, ES_S32 axis);

/**
 * @brief Applies the softmax function to the input tensor.
 *
 * @param[in] input The input tensor.
 * @param[in] output The output tensor containing the softmax-transformed data.
 * @param[in] scaleIn The input tensor scale factor.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_DSP_Softmax(ES_TENSOR_S input, ES_TENSOR_S output, ES_FLOAT scaleIn);

/**
 * @brief Computes the similarity transformation matrix.
 *
 * @param[in] srcPoint      Source points for computing the transformation matrix.
 * @param[in] dstPoint      Destination points for computing the transformation matrix.
 * @param[in] transformMat  Resulting similarity transformation matrix.
 * @return Returns the execution status code: ES_AK_SUCCESS for success, others for failure.
 */
ES_S32 ES_AK_CPU_SimilarityTransform(ES_TENSOR_S srcPoint, ES_TENSOR_S dstPoint, ES_TENSOR_S transformMat);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif